/**
 * This work was created by participants in the DataONE project, and is
 * jointly copyrighted by participating institutions in DataONE. For 
 * more information on DataONE, see our web site at http://dataone.org.
 *
 *   Copyright ${year}
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 * 
 * $Id$
 */
package org.dataone.tidy.dao;

import java.util.List;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;

public class MergeResultRepositoryTest {

    private MergeResultRepository repository;
    private H2RepositoryConfiguration config = new H2RepositoryConfiguration();

    @Test
    public void testSimpleCreateReadTest() {

        repository
                .save(new MergeResult(
                        "pid1cn-dev-ucsb-1:RightsHolder_testGroup.14cn-dev-ucsb-1:RightsHolder_testGroup.14cn-dev-ucsb-1:RightsHolder_testGroup.14cn-dev-ucsb-1:RightsHolder_testGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14estGroup.14cn-dev-ucsb-1:Rp.14cn-dev-ucsb-1:RightsHolder_testGroup.14",
                        MergeStatus.SUCCESS));
        repository.save(new MergeResult("foo", MergeStatus.FAILURE));
        repository.save(new MergeResult("bar", MergeStatus.SUCCESS));

        Iterable<MergeResult> results = repository.findAll();
        System.out.println("Results found with findAll():");
        System.out.println("-------------------------------");
        int count = 0;
        for (MergeResult result : results) {
            System.out.println(result);
            count++;
        }
        System.out.println();
        Assert.assertEquals("Total records found did not match expected.", 3, count);

        results = repository.findByPid("foo");
        System.out.println("Results found with findByPid('foo'):");
        System.out.println("--------------------------------------------");
        count = 0;
        for (MergeResult result : results) {
            System.out.println(result);
            count++;
        }
        Assert.assertEquals("Find by pid found more records than expected", 1, count);
    }

    @Test
    public void testPagingQuery() {
        repository.deleteAll();
        repository.save(new MergeResult("pid1", MergeStatus.SUCCESS));
        repository.save(new MergeResult("foo", MergeStatus.SUCCESS));
        repository.save(new MergeResult("bar", MergeStatus.SUCCESS));
        repository.save(new MergeResult("fu", MergeStatus.SUCCESS));
        repository.save(new MergeResult("pid2", MergeStatus.SUCCESS));

        int pageSize = 1;
        int startPageNumber = 0;

        PageRequest pageRequest = new PageRequest(startPageNumber, pageSize, new Sort(
                Sort.Direction.ASC, "id"));
        Page<MergeResult> pagedResults = repository.findAll(pageRequest);

        Assert.assertEquals("Total elements does not match expected", 5,
                pagedResults.getTotalElements());
        Assert.assertEquals("Total pages does not match expected", 5, pagedResults.getTotalPages());
        Assert.assertTrue("First Page does not agree", pagedResults.isFirstPage());
        Assert.assertFalse("Last Page does not agree", pagedResults.isLastPage());
        Assert.assertEquals("Page size is wrong", 1, pagedResults.getContent().size());

        System.out.println("page " + pagedResults.getNumber() + ": "
                + pagedResults.getContent().get(0));
        while (pagedResults.hasNextPage()) {
            pagedResults = repository.findAll(pagedResults.nextPageable());
            Assert.assertEquals("Page size is wrong", 1, pagedResults.getContent().size());
            System.out.println("page " + pagedResults.getNumber() + ": "
                    + pagedResults.getContent().get(0));
        }
    }

    @Test
    public void testFindByStatusAndRun() {
        repository.deleteAll();

        MergeResult mergeResult = new MergeResult("pid1", MergeStatus.SUCCESS);
        mergeResult.setRunId(new Long(1));
        repository.save(mergeResult);

        mergeResult = new MergeResult("pid1", MergeStatus.SUCCESS);
        mergeResult.setRunId(new Long(2));
        repository.save(mergeResult);

        mergeResult = new MergeResult("pid2", MergeStatus.SUCCESS);
        mergeResult.setRunId(new Long(1));
        repository.save(mergeResult);

        mergeResult = new MergeResult("pid3", MergeStatus.FAILURE);
        mergeResult.setRunId(new Long(1));
        repository.save(mergeResult);

        List<MergeResult> results = repository.findByMergeStatusStrAndRunId(
                MergeStatus.SUCCESS.toString(), new Long(1));

        Assert.assertEquals(2, results.size());
    }

    @Before
    public void setUp() {
        config.initContext();
        repository = config.getMergeResultRepository();
    }

    @After
    public void tearDown() {
        config.closeContext();
    }
}
